package com.supermap.samplecode.data;

import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JToolBar;

import com.supermap.data.Workspace;
import com.supermap.ui.Action;
import com.supermap.ui.GeometrySelectedEvent;
import com.supermap.ui.GeometrySelectedListener;
import com.supermap.ui.MapControl;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.BoxLayout;
import javax.swing.SwingUtilities;

import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.table.DefaultTableModel;

/**
 * <p>
 * Title:编辑历史
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap Objects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：展示编辑历史功能
 * 2、示例数据：安装目录\SampleData\City\Changchun.smwu；
 * 3、关键类型/成员: 
 *      MapControl.TrackedEvent 事件
 *      MapControl.TrackingEvent 事件
 *      MapControl.doKeyDown 方法
 *      MapControl.doMouseDown 方法
 *      MapControl.doMouseMove 方法
 *      MapControl.doMouseWheel 方法
 *      EditHistory.add 方法
 *      EditHistory.undo 方法  
 *      EditHistory.redo 方法    
 * 4、使用步骤：
 *  (1)选择绘制的线类型
 *  (2)在地图中绘制线
 *  (3)选择一个线，修改数据
 *  (4)Ctrl+Z 或者点击"撤消"按钮，查看刚才被添加和修改的地方是否被撤消
 *  (5)Ctrl+Y 或者点击"恢复"按钮，查看刚才撤消的添加和修改是否被恢复
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * </p>
 * 
 */
 
 /**
 * <p>
 * Title:Edit History
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
  * ------------------------------CopyRight----------------------------
 *This file contants the sample code of SuperMap Objects Java.
 * Copyright @2005-2012 SuperMap Software Co., Ltd. All Rights Reserved. 
 * ----------------------------------------------------------------
 * ---------------------SuperMap Objects Java Sample Code Description------------------------
 * 
 * 1. Sample Code Description:This sample demonstrates how to record the history information of the edit operation.
 * 2. Sample Data Location: Installation directory\SampleData\City\Changchun.smwu;
 * 3. Key Classes/Methods:
 * 
 *      MapControl.TrackingEvent event
 *      MapControl.doKeyDown method
 *      MapControl.doMouseDown method
 *      MapControl.doMouseMove method
 *      MapControl.doMouseWheel method
 *      EditHistory.add method
 *      EditHistory.undo method
 *      EditHistory.redo method
 * 4. Procedures:
 *  (1)Choose the type of line. 
 *  (2)Draw line in the map. 
 *  (3)Choose a line and modify its spatial data and field value. 
 *  (4)Press the Ctrl key and Z key simultaneity or click "Cancel" button to cancel the last step. 
 *  (5)Press the Ctrl key and Y key simultaneity or click "Redo" button to redo the step which is canceled just now.
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd
 * </p>
 * 
 */
 
public class MainFrame extends JFrame {

	private static final long serialVersionUID = 1L;

	private JPanel m_jContentPane;

	private JToolBar m_jJToolBar;

	private JPanel m_panelRight;

	private MapControl m_mapControl;

	private JScrollPane m_scrollPaneProperty;

	private JTable m_tableProperty;

	private JButton m_buttonModify;

	private JScrollPane m_scrollPanePoints;

	private JTable m_tablePoints;

	private JPanel m_panelBottom;

	private JPanel m_panelModify;

	private JButton m_buttonApply;

	private JButton m_buttonDelNode;

	private JLabel m_jLabel;

	private JComboBox m_comboBoxPartCount;

	private JButton m_buttonCreateLine;

	private JButton m_buttonCreatePolyline;

	private JButton m_buttonCreateBSpline;

	private JButton m_buttonUndo;

	private JButton m_buttonRedo;

	private Workspace m_workspace;

	private SampleRun m_sampleRun;

	/**
	 * 程序入口点
	 * The entrance of the programme.
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				MainFrame thisClass = new MainFrame();
				thisClass.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				thisClass.setVisible(true);
			}
		});
	}

	/**
	 * 构造函数
	 * The constructor.
	 */
	public MainFrame() {
		super();
		initialize();
	}

	/**
	 * 初始化窗体
	 * Initialize the forms.
	 */
	private void initialize() {

		// 最大化显示窗体
		// Maximum display the form.
		this.setExtendedState(Frame.MAXIMIZED_BOTH);
		this.setSize(800, 500);
		this.setContentPane(getJContentPane());
		this.setTitle(getText("m_title"));
		this.addWindowListener(new WindowAdapter() {

			// 在主窗体加载时，初始化SampleRun类型，来完成功能的展现
			// Initialize a SampleRun object when loading the main forms.
			public void windowOpened(WindowEvent e) {

				m_workspace = new Workspace();
				m_sampleRun = new SampleRun(m_workspace, m_mapControl,
						m_tableProperty, m_tablePoints, m_comboBoxPartCount,
						m_buttonRedo, m_buttonUndo);
				m_mapControl
						.addGeometrySelectedListener(new GeometrySelectedListener() {

							public void geometrySelected(GeometrySelectedEvent e) {
								m_sampleRun.fillComboBox(m_mapControl.getMap()
										.getLayers().get(0).getSelection());

							}

						});
			}

			// 在窗体关闭时，需要释放相关的资源
			// Dispose the resources when closing the forms.
			public void windowClosing(WindowEvent e) {
				m_mapControl.dispose();
				m_workspace.dispose();
			}

		});
	}

	/**
	 * 获取m_jToolBar
	 * Get the m_jToolBar
	 */
	private JToolBar getJToolBar() {
		if (m_jJToolBar == null) {
			m_jJToolBar = new JToolBar();
			m_jJToolBar.setPreferredSize(new java.awt.Dimension(30, 34));
			m_jJToolBar.add(getButtonCreateLine());
			m_jJToolBar.add(getButtonCreatePolyline());
			m_jJToolBar.add(getButtonCreateBSpline());
			m_jJToolBar.add(getButtonUndo());
			m_jJToolBar.add(getButtonRedo());
		}
		return m_jJToolBar;
	}

	/**
	 * 获取m_panelRight
	 * Get the m_panelRight
	 */
	private JPanel getJPanel() {
		if (m_panelRight == null) {
			m_panelRight = new JPanel();
			m_panelRight
					.setLayout(new BoxLayout(getJPanel(), BoxLayout.Y_AXIS));
			m_panelRight.setPreferredSize(new java.awt.Dimension(300, 749));
			m_panelRight.add(getJScrollPane(), null);
			m_panelRight.add(getPanelModify(), null);
			m_panelRight.add(getScrollPanePoints(), null);
			m_panelRight.add(getPanelBottom(), null);
		}
		return m_panelRight;
	}

	/**
	 * 构建MapControl控件
	 * Initialize the MapControl object.
	 */
	private MapControl getMapControl() {
		if (m_mapControl == null) {
			m_mapControl = new MapControl();
		}
		return m_mapControl;
	}

	/**
	 * 获取m_scrollPaneProperty
	 * Get the m_scrollPaneProperty
	 */
	private JScrollPane getJScrollPane() {
		if (m_scrollPaneProperty == null) {
			m_scrollPaneProperty = new JScrollPane();
			m_scrollPaneProperty.setPreferredSize(new java.awt.Dimension(453,
					300));
			m_scrollPaneProperty.setViewportView(getTableProperty());
		}
		return m_scrollPaneProperty;
	}

	/**
	 * 获取m_tableProperty
	 * Get the m_tableProperty
	 */
	private JTable getTableProperty() {
		if (m_tableProperty == null) {
			m_tableProperty = new JTable();
			m_tableProperty.setDragEnabled(false);

			DefaultTableModel model = new DefaultTableModel() {

				private static final long serialVersionUID = 1L;

				public boolean isCellEditable(int row, int column) {

					if (column == 0)
						return false;
					return m_sampleRun.notSysField(row, column);
				}
			};
			model.addColumn(getText("model1"));
			model.addColumn(getText("model2"));
			m_tableProperty.setModel(model);
		}
		return m_tableProperty;
	}

	/**
	 * 修改按钮
	 * The modify button.
	 */
	private JButton getButtonModify() {
		if (m_buttonModify == null) {
			m_buttonModify = new JButton();
			m_buttonModify.setPreferredSize(new java.awt.Dimension(75, 28));
			m_buttonModify
					.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
			m_buttonModify
					.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
			m_buttonModify.setText(getText("m_buttonModify"));
			m_buttonModify.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					m_sampleRun.modifyRecordset();

				}
			});
		}
		return m_buttonModify;
	}

	/**
	 * 获取m_scrollPanePoints
	 * Get the m_scrollPanePoints.
	 */
	private JScrollPane getScrollPanePoints() {
		if (m_scrollPanePoints == null) {
			m_scrollPanePoints = new JScrollPane();
			m_scrollPanePoints.setViewportView(getTablePoints());
		}
		return m_scrollPanePoints;
	}

	/**
	 * 获取m_tablePoints
	 * Get the m_tablePoints
	 */
	private JTable getTablePoints() {
		if (m_tablePoints == null) {
			m_tablePoints = new JTable();
			DefaultTableModel model = new DefaultTableModel() {

				private static final long serialVersionUID = 1L;

				public boolean isCellEditable(int row, int column) {
                      return false;
				}
			};
			model.addColumn(getText("model3"));
			model.addColumn(getText("model4"));
			model.addColumn(getText("model5"));
			m_tablePoints.setModel(model);
		}
		return m_tablePoints;
	}

	/**
	 * 获取m_panelBottom
	 * Get the m_panelBottom
	 */
	private JPanel getPanelBottom() {
		if (m_panelBottom == null) {
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints5.gridy = 0;
			gridBagConstraints5.gridx = 2;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 1;
			gridBagConstraints4.ipadx = 31;
			gridBagConstraints4.ipady = 9;
			gridBagConstraints4.insets = new java.awt.Insets(0, 0, 0, 0);
			gridBagConstraints4.gridy = 0;
			m_jLabel = new JLabel();
			m_jLabel.setText(getText("m_jLabel"));
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 1;
			gridBagConstraints3.anchor = java.awt.GridBagConstraints.CENTER;
			gridBagConstraints3.ipadx = 0;
			gridBagConstraints3.ipady = 0;
			gridBagConstraints3.gridwidth = 1;
			gridBagConstraints3.fill = java.awt.GridBagConstraints.NONE;
			gridBagConstraints3.gridy = 2;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 2;
			gridBagConstraints2.ipadx = 31;
			gridBagConstraints2.ipady = 0;
			gridBagConstraints2.gridwidth = 0;
			gridBagConstraints2.gridheight = 0;
			gridBagConstraints2.insets = new java.awt.Insets(0, 0, 0, 0);
			gridBagConstraints2.gridy = 2;
			m_panelBottom = new JPanel();
			m_panelBottom.setLayout(new GridBagLayout());
			m_panelBottom.add(getButtonApply(), gridBagConstraints2);
			m_panelBottom.add(getButtonDelNode(), gridBagConstraints3);
			m_panelBottom.add(m_jLabel, gridBagConstraints4);
			m_panelBottom.add(getComboBoxPartCount(), gridBagConstraints5);
		}
		return m_panelBottom;
	}

	/**
	 * 获取m_panelModify
	 * Get the m_panelModify
	 */
	private JPanel getPanelModify() {
		if (m_panelModify == null) {
			m_panelModify = new JPanel();
			m_panelModify.add(getButtonModify(), null);
		}
		return m_panelModify;
	}

	/**
	 * 应用按钮
	 * The Apply button
	 */
	private JButton getButtonApply() {
		if (m_buttonApply == null) {
			m_buttonApply = new JButton();
			m_buttonApply.setText(getText("m_buttonApply"));
			m_buttonApply.setName("m_applyButton");
			m_buttonApply.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					m_sampleRun.modifyGeometry();

				}

			});
		}
		return m_buttonApply;
	}

	/**
	 * 删除节点按钮	
	 * The button of Delete Node.
	 */
	private JButton getButtonDelNode() {
		if (m_buttonDelNode == null) {
			m_buttonDelNode = new JButton();
			m_buttonDelNode.setText(getText("m_buttonDelNode"));
			m_buttonDelNode.setName("m_delNodeButton");
			m_buttonDelNode.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					m_sampleRun.delNode();
				}

			});
		}
		return m_buttonDelNode;
	}

	/**
	 * 获取m_comboBoxPartCount	
	 * Get the m_comboBoxPartCount.
	 */
	private JComboBox getComboBoxPartCount() {
		if (m_comboBoxPartCount == null) {
			m_comboBoxPartCount = new JComboBox();
			m_comboBoxPartCount.addItemListener(new ItemListener() {

				public void itemStateChanged(ItemEvent e) {
					m_sampleRun.refreshTable();

				}

			});
		}
		return m_comboBoxPartCount;
	}

	/**
	 * 绘制直线按钮	
	 * The button of Create Line.
	 */
	private JButton getButtonCreateLine() {
		if (m_buttonCreateLine == null) {
			m_buttonCreateLine = new JButton();
			m_buttonCreateLine.setSize(new java.awt.Dimension(62, 30));
			m_buttonCreateLine.setText(getText("m_buttonCreateLine"));
			m_buttonCreateLine.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					m_mapControl.setAction(Action.CREATELINE);

				}

			});
		}
		return m_buttonCreateLine;
	}

	/**
	 * 绘制折线按钮
	 * The button of Create Polyline.
	 */
	private JButton getButtonCreatePolyline() {
		if (m_buttonCreatePolyline == null) {
			m_buttonCreatePolyline = new JButton();
			m_buttonCreatePolyline.setText(getText("m_buttonCreatePolyline"));
			m_buttonCreatePolyline.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					m_mapControl.setAction(Action.CREATE_POLYLINE2_POLYLINE);

				}

			});
		}
		return m_buttonCreatePolyline;
	}

	/**
	 * 绘制B样条曲线按钮
	 * The button of Create BSpline.
	 */
	private JButton getButtonCreateBSpline() {
		if (m_buttonCreateBSpline == null) {
			m_buttonCreateBSpline = new JButton();
			m_buttonCreateBSpline.setText(getText("m_buttonCreateBSpline"));
			m_buttonCreateBSpline.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					m_mapControl.setAction(Action.CREATEBSPLINE);

				}

			});
		}
		return m_buttonCreateBSpline;
	}

	/**
	 * 撤销按钮	
	 * The button of Undo. 
	 */
	private JButton getButtonUndo() {
		if (m_buttonUndo == null) {
			m_buttonUndo = new JButton();
			m_buttonUndo.setText(getText("m_buttonUndo"));
			m_buttonUndo.setEnabled(false);
			m_buttonUndo.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					m_sampleRun.undo();

				}

			});
		}
		return m_buttonUndo;
	}

	/**
	 * 恢复按钮
	 * The button of Redo
	 */
	private JButton getButtonRedo() {
		if (m_buttonRedo == null) {
			m_buttonRedo = new JButton();
			m_buttonRedo.setText(getText("m_buttonRedo"));
			m_buttonRedo.setEnabled(false);
			m_buttonRedo.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					m_sampleRun.redo();

				}
			});
		}
		return m_buttonRedo;
	}

	/**
	 * 获取m_jContentPane
	 * Get the m_jContentPane
	 */
	private JPanel getJContentPane() {
		if (m_jContentPane == null) {
			m_jContentPane = new JPanel();
			m_jContentPane.setLayout(new BorderLayout());
			m_jContentPane.add(getJToolBar(), java.awt.BorderLayout.NORTH);
			m_jContentPane.add(getJPanel(), java.awt.BorderLayout.EAST);
			m_jContentPane.add(getMapControl(), java.awt.BorderLayout.CENTER);
		}
		return m_jContentPane;
	}

	// 根据语言设置text是中文还是英文
	// To change text to English or Chinede by CurrentCulture
	private String getText(String id)
	{
		String text = "";
		if(com.supermap.data.Environment.getCurrentCulture().contentEquals("zh-CN"))
		{
			if(id == "m_title")
			{
				text = "编辑历史";
			} 
			if(id == "model1")
			{
				text = "字段名称";
			} 
			if(id == "model2")
			{
				text = "字段值";
			} 
			if(id == "m_buttonModify")
			{
				text = "修改";
			} 
			if(id == "model3")
			{
				text = "编号";
			} 
			if(id == "model4")
			{
				text = "X字段";
			} 
			if(id == "model5")
			{
				text = "Y字段";
			} 
			if(id == "m_jLabel")
			{
				text = "当前子对象:";
			} 
			if(id == "m_buttonApply")
			{
				text = "应用";
			} 
			if(id == "m_buttonDelNode")
			{
				text = "删除节点";
			} 
			if(id == "m_buttonCreateLine")
			{
				text = "绘制直线";
			} 
			if(id == "m_buttonCreatePolyline")
			{
				text = "绘制折线";
			}
			if(id == "m_buttonCreateBSpline")
			{
				text = "绘制B样条曲线";
			}
			if(id == "m_buttonUndo")
			{
				text = "撤销";
			}
			if(id == "m_buttonRedo")
			{
				text = "恢复";
			}
		}
		else
		{
			if(id == "m_title")
			{
				text = "Edit History";
			} 
			if(id == "model1")
			{
				text = "Field Name";
			} 
			if(id == "model2")
			{
				text = "Field Value";
			} 
			if(id == "m_buttonModify")
			{
				text = "Modify";
			} 
			if(id == "model3")
			{
				text = "Serial Number";
			} 
			if(id == "model4")
			{
				text = "Field X";
			} 
			if(id == "model5")
			{
				text = "Field Y";
			} 
			if(id == "m_jLabel")
			{
				text = "The current object:";
			} 
			if(id == "m_buttonApply")
			{
				text = "Apply";
			} 
			if(id == "m_buttonDelNode")
			{
				text = "Delete Node";
			} 
			if(id == "m_buttonCreateLine")
			{
				text = "Create Line";
			} 
			if(id == "m_buttonCreatePolyline")
			{
				text = "Create Polyline";
			}
			if(id == "m_buttonCreateBSpline")
			{
				text = "Create BSpline";
			}
			if(id == "m_buttonUndo")
			{
				text = "Undo";
			}
			if(id == "m_buttonRedo")
			{
				text = "Redo";
			}
		}
		return text;
	}

}

