package com.supermap.samplecode.data;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.CellEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;

import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import com.supermap.data.CursorType;
import com.supermap.data.DatasetVector;
import com.supermap.data.EditHistory;
import com.supermap.data.EditType;
import com.supermap.data.EncodeType;
import com.supermap.data.FieldInfos;
import com.supermap.data.GeoLine;
import com.supermap.data.Geometry;
import com.supermap.data.GeometryType;
import com.supermap.data.Point2D;
import com.supermap.data.Recordset;
import com.supermap.data.Workspace;
import com.supermap.data.WorkspaceConnectionInfo;
import com.supermap.data.WorkspaceType;
import com.supermap.mapping.Selection;
import com.supermap.ui.Action;
import com.supermap.ui.InteractionMode;
import com.supermap.ui.MapControl;
import com.supermap.ui.TrackMode;
import com.supermap.ui.TrackedEvent;
import com.supermap.ui.TrackedListener;
import com.supermap.ui.TrackingEvent;
import com.supermap.ui.TrackingListener;

/**
 * <p>
 * Title:编辑历史
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap Objects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：展示编辑历史功能
 * 2、示例数据：安装目录\SampleData\City\Changchun.smwu；
 * 3、关键类型/成员: 
 *      MapControl.TrackedEvent 事件
 *      MapControl.TrackingEvent 事件
 *      MapControl.doKeyDown 方法
 *      MapControl.doMouseDown 方法
 *      MapControl.doMouseMove 方法
 *      MapControl.doMouseWheel 方法
 *      EditHistory.add 方法
 *      EditHistory.undo 方法  
 *      EditHistory.redo 方法    
 * 4、使用步骤：
 *  (1)选择绘制的线类型
 *  (2)在地图中绘制线
 *  (3)选择一个线，修改数据
 *  (4)Ctrl+Z 或者点击"撤消"按钮，查看刚才被添加和修改的地方是否被撤消
 *  (5)Ctrl+Y 或者点击"恢复"按钮，查看刚才撤消的添加和修改是否被恢复
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * </p>
 * 
 */
 
 /**
 * <p>
 * Title:Edit History
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
  * ------------------------------CopyRight----------------------------
 *This file contants the sample code of SuperMap Objects Java.
 * Copyright @2005-2012 SuperMap Software Co., Ltd. All Rights Reserved. 
 * ----------------------------------------------------------------
 * ---------------------SuperMap Objects Java Sample Code Description------------------------
 * 
 * 1. Sample Code Description:This sample demonstrates how to record the history information of the edit operation.
 * 2. Sample Data Location: Installation directory\SampleData\City\Changchun.smwu;
 * 3. Key Classes/Methods:
 * 
 *      MapControl.TrackingEvent event
 *      MapControl.doKeyDown method
 *      MapControl.doMouseDown method
 *      MapControl.doMouseMove method
 *      MapControl.doMouseWheel method
 *      EditHistory.add method
 *      EditHistory.undo method
 *      EditHistory.redo method
 * 4. Procedures:
 *  (1)Choose the type of line. 
 *  (2)Draw line in the map. 
 *  (3)Choose a line and modify its spatial data and field value. 
 *  (4)Press the Ctrl key and Z key simultaneity or click "Cancel" button to cancel the last step. 
 *  (5)Press the Ctrl key and Y key simultaneity or click "Redo" button to redo the step which is canceled just now.
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd
 * </p>
 * 
 */
 
public class SampleRun extends MouseAdapter implements TrackedListener,
		TrackingListener, MouseWheelListener, MouseMotionListener {

	private Workspace m_workspace;

	private MapControl m_mapControl;

	private EditHistory m_editHistory;

	private DatasetVector m_datasetVector;

	private Recordset m_recordset;

	private JComboBox m_comboBoxPartCount;

	private GeoLine m_geoLine;

	private Geometry m_geometry;

	private JTable m_tableProperty;

	private JTable m_tablePoints;

	private JButton m_buttonUndo;

	private JButton m_buttonRedo;

	private boolean m_isTracking = false;

	private String tablePropertyNull1 = "";
	private String tablePropertyNull2 = "";
	private String tablePointsNull1 = "";
	private String tablePointsNull2 = "";
	private String tablePointsNull3 = "";

	/**
	 * 根据workspace和mapControl构造 SampleRun对象
	 * Create SampleRun object by the specified workspace and mapControl object.
	 */
	public SampleRun(Workspace workspace, MapControl mapControl,
			JTable propertyTable, JTable pointsTable, JComboBox countComboBox,
			JButton buttonRedo, JButton buttonUndo) {
		m_workspace = workspace;
		m_mapControl = mapControl;
		m_tableProperty = propertyTable;
		m_tablePoints = pointsTable;
		m_comboBoxPartCount = countComboBox;
		m_buttonRedo = buttonRedo;
		m_buttonUndo = buttonUndo;

		m_mapControl.getMap().setWorkspace(workspace);
		m_editHistory = new EditHistory();
		m_editHistory.setCapacity(100);
		InitializeCultureResources();
		initialize();
	}

	/**
	* 初始化多语言资源
	* Initialize CultureResources
	*/
	private void InitializeCultureResources()
	{
		if(com.supermap.data.Environment.getCurrentCulture().contentEquals("zh-CN"))
		{
			tablePropertyNull1 = "字段名称";
			tablePropertyNull2 = "字段值";
			tablePointsNull1 = "编号";
			tablePointsNull2 = "X字段";
			tablePointsNull3 = "Y字段";
		}
		else
		{
			tablePropertyNull1 = "Field Name";
			tablePropertyNull2 = "Field Value";
			tablePointsNull1 = "Serial Number";
			tablePointsNull2 = "Field X";
			tablePointsNull3 = "Field Y";
		}
	}

	/**
	 * 打开需要的工作空间文件及地图
	 * Open the workspace and map.
	 */
	private void initialize() {
		try {
			// 打开工作空间及地图
			// Open the workspace and map.
			WorkspaceConnectionInfo conInfo = new WorkspaceConnectionInfo(
					"../../SampleData/City/Changchun.smwu");
			conInfo.setType(WorkspaceType.SMWU);
			m_workspace.open(conInfo);
			DatasetVector datasetVector = (DatasetVector) m_workspace
					.getDatasources().get(0).getDatasets().get("Busline");
			m_workspace.getDatasources().get(0).getDatasets().delete(
					"CopyBusLine");
			m_datasetVector = (DatasetVector) m_workspace.getDatasources().get(
					0).copyDataset(datasetVector, "CopyBusLine",
					EncodeType.NONE);
			m_mapControl.getMap().getLayers().add(m_datasetVector, true);
			m_mapControl.getMap().viewEntire();
			m_mapControl.getMap().refresh();
			m_mapControl.setTrackMode(TrackMode.TRACK);
			// 调整mapControl的状态
			// Set the mapControl object.
			m_mapControl.setAction(Action.PAN);
			customInteraction();

		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/**
	 * 自定义操作
	 * Custom operation.
	 */
	private void customInteraction() {

		// 设置自定义操作，处理相应的事件
		// Set the custom operation.
		m_mapControl.setInteractionMode(InteractionMode.CUSTOMALL);
		m_mapControl.addMouseListener(this);
		m_mapControl.addMouseMotionListener(this);
		m_mapControl.addMouseWheelListener(this);
		m_mapControl.addTrackingListener(this);
		m_mapControl.addTrackedListener(this);
		m_mapControl.addKeyListener(new KeyAdapter() {

			public void keyPressed(KeyEvent e) {
				if (e.isControlDown()
						&& (e.getKeyCode() == KeyEvent.VK_Z || e.getKeyCode() == KeyEvent.VK_Y)) {
					if (e.getKeyCode() == KeyEvent.VK_Z) {
						undo();
					}
					if (e.getKeyCode() == KeyEvent.VK_Y) {
						redo();
					}

				} else {
					m_mapControl.doKeyDown(e);
				}
			}
		});

	}

	/**
	 * 处理tracked事件
	 * Set the TrackedEvent.
	 */
	public void tracked(TrackedEvent e) {

		DatasetVector vector = (DatasetVector) m_mapControl.getMap()
				.getLayers().get(0).getDataset();
		Recordset recordset = null;
		m_isTracking = false;

		try {
			// 添加tracked的对象到数据集
			// Add the TrackedEvent to the dataset.
			recordset = vector.getRecordset(false, CursorType.DYNAMIC);

			recordset.addNew(e.getGeometry());
			recordset.update();
			m_editHistory.add(EditType.ADDNEW, recordset, true);
			m_buttonUndo.setEnabled(true);
			m_mapControl.getMap().refresh();
		} finally {
			if (recordset != null) {
				recordset.dispose();
			}
		}

	}

	/**
	 * 处理鼠标按下事件
	 * The mouse pressed event.
	 */
	public void mousePressed(MouseEvent e) {
		// 可以根据您的逻辑来确定如何响应事件
		// Set the response event.
		try {
			if (e.getButton() == 3 && e.getClickCount() == 1 && !m_isTracking)
				m_mapControl.setAction(Action.SELECT2);
			else
				m_mapControl.doMouseDown(e);
		} catch (RuntimeException e1) {
			System.out.println(e1.getMessage());
		}
	}

	/**
	 * 处理MouseUp事件
	 * The MouseUp event.
	 */
	public void mouseReleased(MouseEvent e) {
		// 可以根据您的逻辑来确定如何响应事件，这里为默认实现
		// Set the response event.
		m_mapControl.doMouseUp(e);
	}

	/**
	 * 处理MouseWheel事件
	 * The MouseWheel event.
	 */
	public void mouseWheelMoved(MouseWheelEvent e) {
		// 可以根据您的逻辑来确定如何响应事件，这里为默认实现
		m_mapControl.doMouseWheel(e);
	}

	/**
	 * 处理MouseMove事件
	 */
	public void mouseDragged(MouseEvent e) {
		// 可以根据您的逻辑来确定如何响应事件，这里为默认实现
		// Set the response event.
		m_mapControl.doMouseMove(e);
	}

	public void mouseMoved(MouseEvent e) {
		// 可以根据您的逻辑来确定如何响应事件，这里为默认实现
		// Set the response event.
		m_mapControl.doMouseMove(e);
	}

	/**
	 * 恢复操作
	 * Redo the operation.
	 */
	public void redo() {
		if (m_editHistory.getCanRedo()) {
			m_editHistory.redo();
			m_mapControl.getMap().refresh();
			refreshTable();
			if (!m_editHistory.getCanRedo())
				m_buttonRedo.setEnabled(false);
		}

		if (m_editHistory.getCanUndo())
			m_buttonUndo.setEnabled(true);
		else
			m_buttonUndo.setEnabled(false);
	}

	/**
	 * 撤销操作
	 * Undo the operation.
	 */
	public void undo() {
		if (m_editHistory.getCanUndo()) {
			m_editHistory.undo();
			m_mapControl.getMap().refresh();
			refreshTable();
			if (!m_editHistory.getCanUndo())
				m_buttonUndo.setEnabled(false);
		}

		if (m_editHistory.getCanRedo())
			m_buttonRedo.setEnabled(true);
		else
			m_buttonRedo.setEnabled(false);
	}

	/**
	 * 填充m_comboBoxPartCount
	 * Set the m_comboBoxPartCount
	 */
	public void fillComboBox(Selection selection) {
		if (m_recordset != null) {
			m_recordset.dispose();
			m_recordset = null;
		}
		try {
			m_recordset = selection.toRecordset();
			DefaultComboBoxModel comboBoxModel = new DefaultComboBoxModel();
			Geometry geometry = m_recordset.getGeometry();
			m_geoLine = (GeoLine) geometry;
			for (int i = 0; i < m_geoLine.getPartCount(); i++) {
				comboBoxModel.addElement(String.valueOf(i + 1));
			}
			m_comboBoxPartCount.setModel(comboBoxModel);
			refreshTable();

		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/**
	 * 对两个表m_tableProperty,m_tablePoints进行刷新操作
	 * Refresh the m_tableProperty and the m_tablePoints object.
	 */
	public void refreshTable() {
		try {
			int selectIndex = m_comboBoxPartCount.getSelectedIndex();
			m_recordset.moveTo(selectIndex);
			Geometry geometry = m_recordset.getGeometry();
			FieldInfos infos = m_recordset.getFieldInfos();

			CellEditor tableEditorProperty = m_tableProperty.getCellEditor();
			CellEditor tableEditorPoints = m_tablePoints.getCellEditor();
			if (tableEditorProperty != null)
				tableEditorProperty.stopCellEditing();
			if (tableEditorPoints != null)
				tableEditorPoints.stopCellEditing();

			// 如果geometry为空，将两个表变为初始状态
			// Format the table if the geometry object is empty.
			if (geometry == null) {
				DefaultTableModel tablePropertyNull = new DefaultTableModel();
				tablePropertyNull.addColumn(tablePropertyNull1);
				tablePropertyNull.addColumn(tablePropertyNull2);
				m_tableProperty.setModel(tablePropertyNull);

				DefaultTableModel tablePointsNull = new DefaultTableModel();
				tablePointsNull.addColumn(tablePointsNull1);
				tablePointsNull.addColumn(tablePointsNull2);
				tablePointsNull.addColumn(tablePointsNull3);
				m_tablePoints.setModel(tablePointsNull);
				DefaultComboBoxModel comboBoxNull = new DefaultComboBoxModel();
				m_comboBoxPartCount.setModel(comboBoxNull);
				return;
			}

			// 如果geometry不为空，填充两个表
			// Fill the 2 tables if the geometry object is not empty.
			if (geometry.getType() == GeometryType.GEOLINE) {
				m_geoLine = (GeoLine) geometry;
			}
			DefaultTableModel model = (DefaultTableModel) m_tableProperty
					.getModel();
			model.setRowCount(infos.getCount());
			for (int i = 0; i < m_recordset.getFieldCount(); i++) {
				model.setValueAt(infos.get(i).getName(), i, 0);
				model.setValueAt(m_recordset.getFieldValue(i), i, 1);
			}
			m_tableProperty.setModel(model);

			DefaultTableModel tableModel = (DefaultTableModel) m_tablePoints
					.getModel();
			tableModel.setRowCount(m_geoLine.getPart(0).getCount());
			for (int i = 0; i < m_geoLine.getPart(0).getCount(); i++) {
				Point2D point2D = m_geoLine.getPart(0).getItem(i);
				tableModel.setValueAt(String.valueOf(i + 1), i, 0);
				tableModel.setValueAt(String.valueOf(point2D.getX()), i, 1);
				tableModel.setValueAt(String.valueOf(point2D.getY()), i, 2);
			}
			m_tablePoints.setModel(tableModel);
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}

	/**
	 * 为应用按钮实现的方法，将m_comboBoxPartCount对应的对象的记录集添加到m_editHistory
	 * Add the dataset, which the m_comboBoxPartCount related with, to the m_editHistory object.
	 */
	public void modifyGeometry() {
		try {
			int selectIndex = m_comboBoxPartCount.getSelectedIndex();
			m_recordset.moveTo(selectIndex);
			m_editHistory.add(EditType.MODIFY, m_recordset, true);
			m_recordset.edit();
			m_recordset.setGeometry(m_geoLine);
			m_recordset.update();
			m_mapControl.getMap().refresh();
			refreshTable();
			if (m_editHistory.getCanUndo())
				m_buttonUndo.setEnabled(true);
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}

	/**
	 * 为修改按钮实现的方法，使用m_comboBoxPartCount的值修改记录集
	 * Modified the dataset with the value in m_comboBoxPartCount object.
	 */
	public void modifyRecordset() {

		try {
			CellEditor editor = m_tableProperty.getCellEditor();
			if (editor != null)
				editor.stopCellEditing();

			int selectIndex = m_comboBoxPartCount.getSelectedIndex();
			m_recordset.moveTo(selectIndex);
			m_editHistory.add(EditType.MODIFY, m_recordset, true);
			int rowCount = m_tableProperty.getRowCount();
			for (int i = 0; i < rowCount; i++) {
				if (!m_recordset.getFieldInfos().get(i).isSystemField()) {
					m_recordset.edit();
					if (m_tableProperty.getValueAt(i, 1) == null) {
						m_recordset.setFieldValue(i, 0);
					} else
						m_recordset.setFieldValue(i, m_tableProperty
								.getValueAt(i, 1));
					m_recordset.update();
				}
			}
			m_mapControl.getMap().refresh();
			refreshTable();
			if (m_editHistory.getCanUndo())
				m_buttonUndo.setEnabled(true);
		} catch (RuntimeException e) {
			System.out.println(e.getMessage());
		}

	}

	/**
	 * 为删除节点按钮实现的方法
	 * Delete the node.
	 */
	public void delNode() {
		try {
			int pointSelectRows = m_tablePoints.getSelectedRows().length;
			m_recordset.moveTo(m_comboBoxPartCount.getSelectedIndex());
			for (int i = pointSelectRows; i > 0; i--) {
				m_geoLine.getPart(m_comboBoxPartCount.getSelectedIndex())
						.remove(m_tablePoints.getSelectedRow());
				DefaultTableModel tableModel = (DefaultTableModel) m_tablePoints
						.getModel();
				tableModel.removeRow(m_tablePoints.getSelectedRow());
				m_tablePoints.setModel(tableModel);
			}
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}

	/**
	 * 为添加节点按钮实现的方法
	 * Add node.
	 */
	public void addNode() {
		try {
			CellEditor editor = m_tablePoints.getCellEditor();
			if (editor != null)
				editor.stopCellEditing();
			int pointSelectRow = m_tablePoints.getSelectedRow();
			if (pointSelectRow != -1) {
				Point2D pt = new Point2D();
				pt.setX(Double.valueOf(m_tablePoints.getValueAt(pointSelectRow,
						1).toString()));
				pt.setY(Double.valueOf(m_tablePoints.getValueAt(pointSelectRow,
						2).toString()));
				m_geoLine.getPart(0).add(pt);
				DefaultTableModel tableModel = (DefaultTableModel) m_tablePoints
						.getModel();
				Object[] objs = new Object[3];
				objs[0] = tableModel.getRowCount() + 1;
				objs[1] = pt.getX();
				objs[2] = pt.getY();
				tableModel.addRow(objs);
				m_tablePoints.setModel(tableModel);

			}
		} catch (NumberFormatException e) {
			System.out.println(e.getMessage());
		}
	}

	/**
	 * 判断是否为系统字段
	 * Verdict whether it is system field.
	 */
	public boolean notSysField(int row, int column) {
		String name = m_tableProperty.getValueAt(row, 0).toString();
		boolean b = m_recordset.getFieldInfos().get(name).isSystemField();
		return !b;

	}

	/**
	 * 处理tracking事件
	 * TrackingEvent
	 */
	public void tracking(TrackingEvent e) {

		m_isTracking = true;
	}

}

